# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from abc import ABCMeta
from abc import abstractmethod


class IGuiPage(metaclass=ABCMeta):
    """
    [ GUI页面接口 ]
    * 无

    """

    @abstractmethod
    def open(self, *args):
        """
        [ 打开页面 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

    @abstractmethod
    def _show(self, *args):
        """
        [ 渲染页面 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

    @abstractmethod
    def _event(self, *args):
        """
        [ 处理事件 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """




import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

from rains.kit.web import *
from rains.common.run_pool import RunPool


class TaskTest(WebTask):
    """
    测试任务
    """
    user_input: WebElement
    password_input: WebElement
    login_button: WebElement
    create_button: WebElement

    def set_class_starting(self):
        self.user_input = self.plant.element(
            page='超管后台',
            name='邮箱输入框',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div[2]/form/div[1]/div/input'
        )

        self.password_input = self.plant.element(
            page='超管后台',
            name='密码输入框',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div[2]/form/div[2]/div/input'
        )

        self.login_button = self.plant.element(
            page='超管后台',
            name='Login按钮',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div[2]/form/div[4]/div/button'
        )

        self.create_button = self.plant.element(
            page='超管后台',
            name='Login按钮',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div/div[1]/a'
        )

    def set_function_starting(self):
        self.plant.view.to_url('http://manage.dev-tea.cblink.net/')

    def case_01(self):
        """
        登录成功
        """
        self.user_input.input.send('admin@baocai.us')
        self.password_input.input.send('123456')
        self.login_button.mouse.tap()
        return self.create_button.get.in_page() is True


if __name__ == '__main__':
    from rains.common.rains_server import RainsServer
    rains_server = RainsServer()
    rains_server.running()



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

import pathlib

from pathlib import Path


class ConstDirNaming(object):
    """[ 常量 :: 文件夹命名 ]

    * NOT MESSAGE

    """

    LOGS: str = 'Logs'
    """ [ 常量 :: 文件夹命名 :: 日志 ] """

    DATA: str = 'Data'
    """ [ 常量 :: 文件夹命名 :: 数据 ] """


class ConstFileNaming(object):
    """[ 常量 :: 文件命名 ]

    * NOT MESSAGE

    """

    DB: str = 'RainsDatabase.db'
    """ [ 常量 :: 文件命名 :: 数据库 ] """


class ConstPath(object):
    """[ 常量 :: 路径 ]

    * NOT MESSAGE

    """

    ROOT: Path = pathlib.Path().cwd()
    """ [ 常量 :: 路径 :: 项目所在目录 ] """

    LOGS: Path = ROOT.joinpath(ConstDirNaming.LOGS)
    """ [ 常量 :: 路径 :: 项目日志目录 ] """

    DATA: Path = ROOT.joinpath(ConstDirNaming.DATA)
    """ [ 常量 :: 路径 :: 项目数据目录 ] """

    DATA_DB: Path = DATA.joinpath(ConstFileNaming.DB)
    """ [ 常量 :: 路径 :: 项目数据库文件 ] """



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


class ConstApiStateCode(object):
    """
    [ 常量 :: API 状态码 ]

    * NOT MESSAGE

    """

    SUCCESSFUL: int = 200
    """ [ 常量 :: API 状态码 :: 成功 ] """

    UNSUCCESSFUL: int = 500
    """ [ 常量 :: API 状态码 :: 失败 ] """


class ConstApiStateMsg(object):
    """
    [ 常量 :: API 状态信息 ]

    * NOT MESSAGE

    """

    SUCCESSFUL: str = '成功'
    """ [ 常量 :: API 状态信息 :: 成功 ] """

    UNSUCCESSFUL: str = '失败'
    """ [ 常量 :: API 状态信息 :: 失败 ] """



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


class ConstSwitchState(object):
    """
    [ 常量 :: 开关状态 ]

    * NOT MESSAGE

    """

    ON: str = 'CONST_SWITCH_STATE_ON'
    """ [ 常量 :: 开关状态 :: 开 ] """

    OFF: str = 'CONST_SWITCH_STATE_OFF'
    """ [ 常量 :: 开关状态 :: 关 ] """


class ConstLifecycleState(object):
    """
    [ 常量 :: 生命周期状态 ]

    * NOT MESSAGE

    """

    RUN: str = 'CONST_LIFECYCLE_STATE_RUN'
    """ [ 常量 :: 生命周期状态 :: 运行 ] """

    END: str = 'CONST_LIFECYCLE_STATE_END'
    """ [ 常量 :: 生命周期状态 :: 停止 ] """

    ERR: str = 'CONST_LIFECYCLE_STATE_ERR'
    """ [ 常量 :: 生命周期状态 :: 异常 ] """



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from selenium.webdriver.common.by import By

from rains.common.log import Log

from rains.kit.web.web_engine import WebEngine
from rains.kit.web.packaging.web_plant_view import WebPlantView
from rains.kit.web.packaging.web_plant_element import WebPlantElement
from rains.kit.web.packaging.web_element_structure import WebElementStructure


BY = By


class WebPlant(object):
    """
    [ Web工厂构建类 ]
    * 工厂构建类提供了对浏览器视图与控件元素的封装。
    * 浏览器视图: 封装了浏览器本身的功能，例如访问URL、管理标签页、控制弹窗、上传文件等。
    * 控件元素：封装了对页面元素的操作功能，如控制鼠标、控制输入框、下拉框、判断元素是否存在等。

    """

    _log = Log()
    """ 日志服务 """

    _web_engine: WebEngine
    """ 引擎对象 """

    def __init__(self, web_engine: WebEngine):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * web_engine (WebEngine): WEB引擎对象

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._web_engine = web_engine

    @property
    def engine(self):
        return self._web_engine

    @property
    def driver(self):
        return self._web_engine.driver

    @property
    def view(self) -> WebPlantView:
        """
        [ Web视图 ]
        * 提供针对浏览器本身的操作函数，例如访问URL，最大化窗口，截图PNG等等。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (WebPlantView) : Web视图对象

        """

        return WebPlantView(self._web_engine)

    def element(self,
                page: str,
                name: str,
                by_key: str,
                by_value: str,
                anchor_by_key: str = None,
                anchor_by_value: str = None,
                anchor_location_id: int = None) -> WebPlantElement:
        """
        [ Web元素控件 ]
        * 基于元素特征构建指向该元素的元素控件对象。
        * 元素控件对象封装了针对元素的一些操作元素，例如鼠标点击、键盘输入等等。

        [ 必要参数 ]
        * page (str) : 页面名称
        * name (str) : 组件名称
        * by_key (str) : 元素定位策略
        * by_value (str) : 元素定位策略对应的值

        [ 可选参数 ]
        * anchor_by_key (str) : 锚点元素定位策略
        * anchor_by_value (str) : 锚点元素定位策略
        * anchor_location_id (str) : 锚点元素定位ID

        [ 返回内容 ]
        * (WebPlantElement) : Web元素控件对象

        """

        element_structure = WebElementStructure(page, name, by_key, by_value,
                                                anchor_by_key, anchor_by_value, anchor_location_id)

        return WebPlantElement(self._web_engine, element_structure)



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


def singleton_pattern(cls, *args, **kwargs):
    """
    [ 单例模式装饰器 ]

    * NOT MESSAGE

    """

    instances = {}

    def _singleton_pattern():
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return _singleton_pattern


CONST_SINGLETON_PATERN = singleton_pattern
"""[ 常量单例模式装饰器 ]

* NOT MESSAGE

"""

CLASS_SINGLETON_PATERN = singleton_pattern
"""[ 类单例模式装饰器 ]

* NOT MESSAGE

"""



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


import random

from rains.common.i.i_task import ITask
from rains.common.const import ConstTaskType

from rains.kit.mini.mini_plant import MiniPlant


class MiniTask(ITask):
    """
    [ 小程序任务集 ]
    * 无

    """

    _task_id: int = random.randint(10000, 99999)
    """ 任务编号 """

    _type: str = ConstTaskType.CODE
    """ 任务类型 """

    plant: MiniPlant
    """ 工厂对象 """

    def set_class_starting(self):
        pass

    def set_class_ending(self):
        pass

    def set_function_starting(self):
        pass

    def set_function_ending(self):
        pass



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from rains.kit.web.web_task import WebTask
from rains.kit.web.web_core import WebCore
from rains.kit.web.web_core import BROWSER_TYPE
from rains.kit.web.web_plant import BY
from rains.kit.web.web_plant import WebPlant
from rains.kit.web.web_plant import WebPlantElement
from rains.kit.web.web_model import WebModel

WebElement = WebPlantElement

__all__ = ['WebTask', 'WebCore', 'BROWSER_TYPE', 'BY', 'WebPlant', 'WebElement', 'WebModel']



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

from rains.api.common import RAINS_DB
from rains.api.common import RAINS_SQL
from rains.api.common import URL_PREFIX
from rains.api.common import jsonify
from rains.api.common import request
from rains.api.common import Blueprint
from rains.api.common import ServerParameterHandler

from rains.common.log import Log


LOG: Log = Log()

# 测试接口蓝图
test_blueprint = Blueprint('test', __name__)

# 临时用户数据
USERS = {
    'admin': {
        'userId': 1,
        'username': 'admin',
        'realName': 'Rains Admin',
        'avatar': 'http://mms0.baidu.com/it/u=2163504278,3640533387&fm=253&app=138&f=JPEG&fmt=auto&q=75?w=500&h=707',
        'desc': 'manager',
        'password': 'admin',
        'token': 'fakeToken1',
        'homePath': '/dashboard/workbench',
        'roles': [
            {
                'roleName': 'Super Admin',
                'value': 'super'
            },
        ],
    }
}

@test_blueprint.route(f'{URL_PREFIX}/login', methods=['POST'])
def get() -> jsonify:
    try:
        paras: dict = ServerParameterHandler.analysis_request_parameter(keys=['username', 'password'])

        if paras['username'] in USERS.keys():
            user = USERS[paras['username']]
            if user['password'] == paras['password']:
                result = {
                    'roles': user['roles'],
                    'userId': user['userId'],
                    'username': user['username'],
                    'token': user['token'],
                    'realName': user['realName'],
                    'desc': user['desc'],
                }
                return jsonify({
                    'code': 0,
                    'result': result,
                    'message': 'ok',
                    'type': 'success'
                })
        else:
            return jsonify({
                'code': -1,
                'result': None,
                'message': 'Incorrect account or password！',
                'type': 'error'
            })

    except BaseException as err:
        return ServerParameterHandler.unsuccessful(f'{ err }')


@test_blueprint.route(f'{URL_PREFIX}/getUserInfo', methods=['GET'])
def get_user_info() -> jsonify:
    try:
        token = request.headers['Authorization']

        if token:
            for k, v in USERS.items():
                if token == v['token']:
                    return jsonify({
                        'code': 0,
                        'result': v,
                        'message': 'ok',
                        'type': 'success'
                    })

        else:
            return jsonify({
                'code': -1,
                'result': None,
                'message': 'Invalid token!',
                'type': 'error'
            })

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')


@test_blueprint.route(f'{URL_PREFIX}/logout', methods=['GET'])
def logout() -> jsonify:
    try:
        token = request.headers['Authorization']

        if token:
            for k, v in USERS.items():
                if token == v['token']:
                    return jsonify({
                        'code': 0,
                        'result': v,
                        'message': 'Token has been destroyed',
                        'type': 'success'
                    })

        else:
            return jsonify({
                'code': -1,
                'result': None,
                'message': 'Invalid token!',
                'type': 'error'
            })

    except BaseException as err:
        return ServerParameterHandler.unsuccessful(f'{ err }')


@test_blueprint.route(f'{URL_PREFIX}/table/getDemoList', methods=['GET'])
def table() -> jsonify:
    try:
        page = request.args.get('page')
        page_size = request.args.get('pageSize')
        token = request.headers['Authorization']

        paras = {
            'page': page,
            'number': page_size
        }

        # 获取服务器数据
        db_r = RAINS_DB.read(RAINS_SQL.task.get_info_all(paras))
        task_all_count = RAINS_DB.read(RAINS_SQL.task.get_count_all())[0][0]
        r_task = []
        for t in db_r:
            task = {}
            task.update({'tid': t[0]})
            task.update({'name': t[1]})
            task.update({'remark': t[2]})
            task.update({'created_date': str(t[3])})
            task.update({'state': t[4]})
            task.update({'start_time': t[5]})
            task.update({'end_time': t[6]})
            task.update({'spend_time_s': t[7]})
            r_task.append(task)

        if token:
            for k, v in USERS.items():
                if token == v['token']:
                    return jsonify({
                        'code': 0,
                        'result': {
                            'items': r_task,
                            'total': task_all_count
                        },
                        'message': 'ok',
                        'type': 'success'
                    })

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

from rains.db.blueprint.sql_task import SqlTask
from rains.db.blueprint.sql_case import SqlCase
from rains.db.blueprint.sql_config import SqlConfig
from rains.db.blueprint.sql_run_environment import SqlRunEnvironment


class RainsSql(object):
    """
    [ SQL语句映射类 ]

    * NOT MESSAGE

    """

    @property
    def run_environment(self) -> SqlRunEnvironment:
        """
        [ 运行环境相关 ]

        * NOT MESSAGE

        Returns:
            SqlRunEnvironment: [ SQL语句 :: 运行环境相关 ]

        """

        return SqlRunEnvironment

    @property
    def config(self) -> SqlConfig:
        """
        [ 配置相关 ]

        * NOT MESSAGE

        Returns:
            SqlRunEnvironment: [ SQL语句 :: 配置相关 ]

        """

        return SqlConfig

    @property
    def task(self) -> SqlTask:
        """
        [ 任务相关 ]

        * NOT MESSAGE

        Returns:
            SqlTask: [ SQL语句 :: 任务相关 ]

        """

        return SqlTask

    @property
    def case(self) -> SqlCase:
        """
        [ 用例相关 ]

        * NOT MESSAGE

        Returns:
            SqlCase: [ SQL语句 :: 用例相关 ]

        """

        return SqlCase




import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

from rains.common.rains_server import RainsServer
rains_server = RainsServer()
rains_server.running()



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from rains.common.log import Log

from rains.kit.mini.mini_engine import MiniEngine
from rains.kit.mini.packaging.mini_element_structure import MiniElementStructure

from rains.kit.mini.packaging.mini_plant_view import MiniPlantView
from rains.kit.mini.packaging.mini_plant_element import MiniPlantElement


class MiniPlant(object):
    """
    [ 小程序工厂构建类 ]
    * 工厂构建类提供了对小程序视图与控件元素的封装。

    """

    _log = Log()
    """ [ 日志服务 ] """

    _mini_engine: MiniEngine
    """ [ 引擎对象 ] """

    def __init__(self, mini_engine: MiniEngine):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * mini_engine (MiniEngine) : 小程序引擎对象

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._mini_engine = mini_engine

        # 如果引擎中的驱动未启动，则启动
        if not self._mini_engine.driver:
            self._mini_engine.run()

    @property
    def engine(self):
        return self._mini_engine

    @property
    def driver(self):
        return self._mini_engine.driver

    @property
    def view(self) -> MiniPlantView:
        """
        [ 小程序视图 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (MiniPlantView) : 小程序视图对象

        """

        return MiniPlantView(self._mini_engine)

    def element(self,
                page: str,
                name: str,
                selector: str) -> MiniPlantElement:
        """
        [ 小程序元素控件 ]
        * 基于元素特征构建指向该元素的元素控件对象。
        * 元素控件对象封装了针对元素的一些操作元素，例如鼠标点击、键盘输入等等。

        [ 必要参数 ]
        * page (str) : 页面名称
        * name (str) : 组件名称
        * selector (str) : css选择器或以/或//开头的xpath

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (MiniPlantElement) : 小程序元素对象

        """

        element_structure = MiniElementStructure(
            page=page,
            name=name,
            selector=selector
        )

        return MiniPlantElement(self._mini_engine, element_structure)



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from rains.kit.web.web_plant import WebPlant


class WebModel(object):
    """
    [ Web模型 ]
    * 无

    """

    plant: WebPlant

    def __init__(self, web_plant: WebPlant):
        """
        [ 初始化 ]
        * 构建页面模型主要由 Plant 构建类完成，实例化页面模型时需传递 Plant 实例。

        [ 必要参数 ]
        * web_plant (WebPlant) : Web工厂构建类

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self.plant = web_plant





class T(object):

    a = f"""

    CREATE TABLE IF NOT EXISTS 1
    (
        id                                               INTEGER PRIMARY KEY,
        2   BLOB NOT NULL,
        3  INT NOT NULL,
        4  INT NOT NULL
    );

    """

    b = f"""

    CREATE TABLE IF NOT EXISTS 1
    (
        id                                               INTEGER PRIMARY KEY,
        2   BLOB NOT NULL,
        3  INT NOT NULL,
        4  INT NOT NULL
    );

    """

ls = []
for k, v in T.__dict__.items():
    if '_' in k:
        continue
    ls.append(v)

print(len(ls))




# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

from rains.api.common import RAINS_DB
from rains.api.common import RAINS_SQL
from rains.api.common import URL_PREFIX
from rains.api.common import jsonify
from rains.api.common import Blueprint
from rains.api.common import ServerParameterHandler


# 运行环境接口蓝图
run_environment_blueprint = Blueprint('run_environment', __name__)


@run_environment_blueprint.route(f'{URL_PREFIX}/run_environment/get', methods=['GET'])
def get() -> jsonify:
    """
    [ 查询运行环境 ]

    * NOT MESSAGE

    Returns:
        jsonify: [ 返回前端的 Json 数据结构 ]

    """

    try:
        data = RAINS_DB.lock_read(RAINS_SQL.run_environment.get())[0]

        return ServerParameterHandler.successful({
            'pool_run_state': bool(data[1]),
            'core_sign_count': data[2],
            'task_sign_count': data[3]
        })

    except BaseException as err:
        return ServerParameterHandler.unsuccessful(f'{ err }')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = ['Log']


import time
import logging

from rains.const.const_file import ConstPath
from rains.common.decorator import CLASS_SINGLETON_PATERN


# 日志服务 :: 记录器日志级别
_CONFIG_LOGGING_LEVEL = 'DEBUG'

# 日志服务 :: 默认流处理器的日志级别
_CONFIG_BASE_HANDLE_LEVEL = 'INFO'

# 日志服务 :: 文件流处理器的日志级别
_CONFIG_FILE_HANDLE_LEVELS = ['DEBUG', 'INFO', 'ERROR', 'WARNING']

# 日志服务 :: 日志输出格式对象
_CONFIG_OUTPUT_STRUCTURE = logging.Formatter('[%(asctime)s] [%(levelname)7s] %(message)s')


@CLASS_SINGLETON_PATERN
class Log(object):
    """
    [ 日志服务 ]

    * NOT MESSAGE

    """

    _logger: logging.Logger
    """ [ 记录器对象 ] """

    def __init__(self):
        """
        [ 日志服务 ]

        * 初始化。

        Raises:
            CreationLogLivingError: [ 创建日志实例异常 ]

        """

        try:
            # 创建记录器
            self._logger = logging.getLogger()
            self._logger.setLevel(_CONFIG_LOGGING_LEVEL)
            self._logger.handlers.clear()

            # 创建默认流处理器
            self._creation_base_handle()
            # 创建文件流处理器
            self._creation_file_handle()

        except BaseException as err:
            raise CreationLogLivingError(str(err))

    def debug(self, message: str):
        """
        [ 记录 DEBUG ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的日志信息 ]

        """

        self._logger.debug(message)

    def info(self, message):
        """
        [ 记录 INFO ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的日志信息 ]

        """

        self._logger.info(message)

    def warning(self, message):
        """
        [ 记录 WARNING ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的日志信息 ]

        """

        self._logger.warning(message)

    def error(self, message):
        """
        [ 记录 ERROR ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的日志信息 ]

        """

        self._logger.error(message)

    def critical(self, message):
        """
        [ 记录 CRITICAL ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的日志信息 ]

        """

        self._logger.critical(message)

    def _creation_base_handle(self):
        """
        [ 创建默认流处理器 ]

        * NOT MESSAGE

        """

        # 创建处理器
        base_handle = logging.StreamHandler()
        # 配置日志输出格式
        base_handle.setFormatter(_CONFIG_OUTPUT_STRUCTURE)
        # 设置处理器日志等级
        base_handle.setLevel(_CONFIG_BASE_HANDLE_LEVEL)
        # 注册处理器
        self._logger.addHandler(base_handle)

    def _creation_file_handle(self):
        """
        [ 创建文件流处理器 ]

        * NOT MESSAGE
        
        """

        # 创建当前项目的日志根目录
        if not ConstPath.LOGS.is_dir():
            ConstPath.LOGS.mkdir()

        # 创建当前项目的当天日志存放目录
        date = time.strftime('%Y-%m-%d', time.localtime())
        path = ConstPath.LOGS.joinpath(date)
        if not path.is_dir():
            path.mkdir()

        # 创建文件处理器
        for v in _CONFIG_FILE_HANDLE_LEVELS:

            # 创建处理器
            file_handle = logging.FileHandler(f'{path.joinpath(f"{ v }.log")}')
            # 配置日志输出格式
            file_handle.setFormatter(_CONFIG_OUTPUT_STRUCTURE)
            # 设置处理器的日志等级
            file_handle.setLevel(v)
            # 注册处理器
            self._logger.addHandler(file_handle)


class CreationLogLivingError(Exception):

    def __init__(self, message: str):
        """
        [ 创建日志实例异常 ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的错误信息 ]

        """

        self.message = message
    
    def __str__(self) -> str:

        return f"创建日志实例异常:: { self.message }"



# !/usr/bin/env python
# coding:utf-8

"""
[ API ]

* NOT MESSAGE

"""

__all__ = [
    'rains_app',
    'RAINS_DB',
    'RAINS_SQL',
    'URL_PREFIX',
    'Response',
    'request',
    'jsonify',
    'Blueprint',
    'ServerParameterHandler',
]

from .app import rains_app
from .common import RAINS_DB
from .common import RAINS_SQL
from .common import URL_PREFIX
from .common import Response
from .common import request
from .common import jsonify
from .common import Blueprint
from .common import ServerParameterHandler



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from abc import ABCMeta
from abc import abstractmethod


class ICore(metaclass=ABCMeta):
    """
    [ 核心接口 ]
    * 无

    """

    @abstractmethod
    def start_task(self, task):
        """
        [ 开始执行 task ]
        * 接收一个 task，并执行该 task。

        [ 必要参数 ]
        * task (object): 任务对象，可以是实现 ITask 接口的任务类，或者是符合JSON译文标准的字典

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


class ConstProjectType(object):
    """
    [ 常量 :: 项目类型 ]

    * NOT MESSAGE

    """

    WEB: str = 'CONST_PROJECT_TYPE_FROM_WEB'
    """ [ 常量 :: 项目类型 :: WEB ] """

    MINI: str = 'CONST_PROJECT_TYPE_FROM_MINI'
    """ [ 常量 :: 项目类型 :: 微信小程序 ] """


class ConstTaskType(object):
    """
    [ 常量 :: 任务类型 ]

    * NOT MESSAGE

    """

    CODE: str = 'CONST_TASK_TYPE_FROM_CODE'
    """ [ 常量 :: 任务类型 :: 代码 ] """

    JSON: str = 'CONST_TASK_TYPE_FROM_JSON'
    """ [ 常量 :: 任务类型 :: JSON 译文 ] """



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from rains.api.common import *


app_blueprint_case = Blueprint('case', __name__)
""" [ 用例蓝图 ] """


@app_blueprint_case.route('/case/cases', methods=['POST'])
def cases() -> Response:
    """
    [ 获取用例列表 ]
    * 接收一个任务ID, 返回该任务的所有用例.

    [ 必要参数 ]
    * tid     (int) : 任务ID

    [ 可选参数 ]
    * state   (str) : 状态
    * page    (int) : 页数
    * number  (int) : 查询数量

    [ 返回内容 ]
    * Response 对象

    """

    try:
        # 解析请求参数
        paras: dict = ServerParameterHandler.analysis_request_parameter('tid', 'state', 'page', 'number')
        # 获取服务器数据
        base_case_list: list = DB.read(SQL.case.get_info_from_tid(paras))

        # 加工数据
        # 将用例信息中的运行步骤转换成列表然后重新拼接数据
        new_case_list: list = []
        for case_info in base_case_list:
            new_case_info: list = []
            number: int = 0
            for info in case_info:
                if number == 9:
                    info = info.split('\n')
                new_case_info.append(info)
                number += 1
            new_case_list.append(new_case_info)

        return ServerParameterHandler.successful({'tid': paras['tid'], 'cases': new_case_list})

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


import time

from minium import Page
from minium import Minium

from rains.common.log import Log
from rains.kit.mini.packaging.mini_element_structure import MiniElementStructure


class MiniFindHandle(object):
    """
    [ 小程序元素定位器 ]
    * 基于 minium 原生定位函数的封装。

    """

    _log = Log()
    """ [ 日志服务 ] """

    _driver: Minium
    """ [ 原生驱动对象 ] """

    def __init__(self, base_driver: Minium):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * base_driver (WXMinium) : 小程序驱动对象

        """

        self._driver = base_driver

    def find(self,
             mini_element_structure: MiniElementStructure,
             wait: int = 15) -> list:
        """
        [ 元素定位 ]
        * 根据参数描述从当前页面上定位元素列表。

        [ 必要参数 ]
        * mini_element_structure (str) : 小程序元素结构体

        [ 可选参数 ]
        * wait (int) : 查找元素所等待的显式等待时长

        [ 返回内容 ]
        * 已获取定位的元素列表

        """

        try:

            time.sleep(0.2)
            page: Page = self._driver.app.get_current_page()

            return page.get_elements(
                selector=mini_element_structure.selector,
                max_timeout=wait,
            )

        except BaseException as e:
            raise Exception(f'定位元素时发生了异常:: { e } !')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

__all__ = [
    'RainsDbParameterHandler',
    'ConstLifecycleState',
    'ConstDbTableNaming',
    'ConstDbRunEnvironmentNaming',
    'ConstDbConfigNaming',
    'ConstDbTaskNaming',
    'ConstDbCaseNaming'
]


from rains.const.const_state import ConstLifecycleState
from rains.const.const_db import ConstDbTableNaming
from rains.const.const_db import ConstDbRunEnvironmentNaming
from rains.const.const_db import ConstDbConfigNaming
from rains.const.const_db import ConstDbTaskNaming
from rains.const.const_db import ConstDbCaseNaming


class RainsDbParameterHandler(object):
    """
    [ 数据库参数处理程序 ]

    * 该类用于解析前端请求参数，以及拼接返回给前端的 Json 数据结构。

    """

    @staticmethod
    def get_desc_limit(paras: dict) -> str:
        """
        [ 获取数据返回量区间限制 ]

        * 默认是获取 1 页， 10 条数据。

        Args:
            paras (dict): [ 参数字典 ]

        ParasMustKeys:
            page (int): [ 页数 ]
            number (int): [ 单页数据量 ]

        Returns:
            str: [ 拼接完成的 SQL LIMIT 语句参数 ]

        """

        # 获取参数
        paras = {} if not paras else paras
        page = 1 if 'page' not in paras.keys() else int(paras['page'])
        number = 10 if 'number' not in paras.keys() else int(paras['number'])

        # 计算偏移量
        limit_begin = 0
        if page > 1:
            limit_begin = (page - 1) * number

        return f'{ limit_begin }, { number }'



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

from flask import Flask
from .blueprint import __all__ as blueprint_all


# 创建应用
rains_app = Flask(__name__)
""" [ RainsApi 应用 ] """

# 更新配置
rains_app.config.update({'JSON_AS_ASCII': False})

# 注册蓝图
for blueprint in blueprint_all:
    print(blueprint)
    rains_app.register_blueprint(blueprint)



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

import random

from flask import jsonify
from flask import request

from rains.api.common import *
from rains.common.run_pool import RunPool

from rains.kit.web import *


app_blueprint_test = Blueprint('test', __name__)
""" [ 用例蓝图 ] """


USERS = {
    'admin': {
        'userId': 1,
        'username': 'admin',
        'realName': 'Rains Admin',
        'avatar': 'http://mms0.baidu.com/it/u=2163504278,3640533387&fm=253&app=138&f=JPEG&fmt=auto&q=75?w=500&h=707',
        'desc': 'manager',
        'password': 'admin',
        'token': 'fakeToken1',
        'homePath': '/dashboard/workbench',
        'roles': [
            {
                'roleName': 'Super Admin',
                'value': 'super'
            },
        ],
    }
}


TASKS = []
for i in range(200):
    TASKS.append({
        'id': i,
        'name': f'任务名称很长阿-{i}',
        'remark': '虚假的任务注释也很长阿',
        'execute_date': '2021-12-12',
        'is_completed': '已完成',
        'start_time': '12:12',
        'end_time': '12:12',
        'spend_time_s': random.randint(20, 100)
    })


@app_blueprint_test.route('/test', methods=['GET'])
def test() -> Response:
    try:
        token = request.headers['Authorization']
        if token:
            for k, v in USERS.items():
                if token == v['token']:

                    # core = WebCore()
                    # plant = WebPlant(core.engine)
                    # plant.view.to_url('https://www.baidu.com/')
                    #
                    # plant.element(
                    #     page='百度首页',
                    #     name='搜索框',
                    #     by_key='xpath',
                    #     by_value='//*[@id="kw"]'
                    # ).input.send('rains nice!')
                    #
                    # plant.element(
                    #     page='百度首页',
                    #     name='百度一下button',
                    #     by_key='xpath',
                    #     by_value='//*[@id="su"]'
                    # ).mouse.tap()
                    #
                    # plant.engine.end()
                    # del core
                    # del plant

                    RunPool(
                        cores=[WebCore()],
                        tasks=[TaskTest]
                    ).running()

                    return jsonify({
                        'code': 0,
                        'result': {'data': '脚本已经执行完毕！'},
                        'message': 'ok',
                        'type': 'success'
                    })

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')


@app_blueprint_test.route('/api/login', methods=['POST'])
def login() -> Response:
    try:
        paras: dict = ServerParameterHandler.analysis_request_parameter('username', 'password')

        if paras['username'] in USERS.keys():
            user = USERS[paras['username']]
            if user['password'] == paras['password']:
                result = {
                    'roles': user['roles'],
                    'userId': user['userId'],
                    'username': user['username'],
                    'token': user['token'],
                    'realName': user['realName'],
                    'desc': user['desc'],
                }
                return jsonify({
                    'code': 0,
                    'result': result,
                    'message': 'ok',
                    'type': 'success'
                })
        else:
            return jsonify({
                'code': -1,
                'result': None,
                'message': 'Incorrect account or password！',
                'type': 'error'
            })

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')


@app_blueprint_test.route('/api/getUserInfo', methods=['GET'])
def get_user_info() -> Response:
    try:
        token = request.headers['Authorization']

        if token:
            for k, v in USERS.items():
                if token == v['token']:
                    return jsonify({
                        'code': 0,
                        'result': v,
                        'message': 'ok',
                        'type': 'success'
                    })

        else:
            return jsonify({
                'code': -1,
                'result': None,
                'message': 'Invalid token!',
                'type': 'error'
            })

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')


@app_blueprint_test.route('/api/logout', methods=['GET'])
def logout() -> Response:
    try:
        token = request.headers['Authorization']

        if token:
            for k, v in USERS.items():
                if token == v['token']:
                    return jsonify({
                        'code': 0,
                        'result': v,
                        'message': 'Token has been destroyed',
                        'type': 'success'
                    })

        else:
            return jsonify({
                'code': -1,
                'result': None,
                'message': 'Invalid token!',
                'type': 'error'
            })

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')


@app_blueprint_test.route('/api/table/getDemoList', methods=['GET'])
def table() -> Response:
    try:
        page = request.args.get('page')
        page_size = request.args.get('pageSize')
        token = request.headers['Authorization']
        number = int(page) * int(page_size)

        paras = {
            'page': page,
            'number': request.args.get('pageSize')
        }

        # 获取服务器数据
        db_r = DB.read(SQL.task.get_info_all(paras))
        task_all_count = DB.read(SQL.task.get_count_all())[0][0]
        r_task = []
        for t in db_r:
            task = {}
            task.update({'id': t[0]})
            task.update({'name': t[1]})
            task.update({'remark': t[2]})
            task.update({'execute_date': str(t[3])})
            task.update({'is_completed': t[4]})
            task.update({'start_time': t[5]})
            task.update({'end_time': t[6]})
            task.update({'spend_time_s': t[7]})
            r_task.append(task)

        if token:
            for k, v in USERS.items():
                if token == v['token']:
                    return jsonify({
                        'code': 0,
                        'result': {
                            # 'items': TASKS[number-int(page_size):number:],
                            'items': r_task,
                            'total': task_all_count
                        },
                        'message': 'ok',
                        'type': 'success'
                    })

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')


class TaskTest(WebTask):
    """
    测试任务
    """
    user_input: WebElement
    password_input: WebElement
    login_button: WebElement
    create_button: WebElement

    def set_class_starting(self):
        self.user_input = self.plant.element(
            page='超管后台',
            name='邮箱输入框',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div[2]/form/div[1]/div/input'
        )

        self.password_input = self.plant.element(
            page='超管后台',
            name='密码输入框',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div[2]/form/div[2]/div/input'
        )

        self.login_button = self.plant.element(
            page='超管后台',
            name='Login按钮',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div[2]/form/div[4]/div/button'
        )

        self.create_button = self.plant.element(
            page='超管后台',
            name='登录成功判断元素',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div/div[1]/a'
        )

    def set_function_starting(self):
        self.plant.view.to_url('http://manage.dev-tea.cblink.net/')

    def case_01(self):
        """
        登录
        """
        self.user_input.input.send('admin@baocai.us')
        self.password_input.input.send('123456')
        self.login_button.mouse.tap()
        return self.create_button.get.in_page() is True



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = ['RainsServer']


import gc
import time
import psutil

from multiprocessing.dummy import Pool
from multiprocessing.dummy import Manager
from multiprocessing.dummy import Lock
from multiprocessing.dummy import Queue

from tornado.wsgi import WSGIContainer
from tornado.httpserver import HTTPServer
from tornado.ioloop import IOLoop

from rains.common.log import Log

from rains.api import rains_app
from rains.api import request
from rains.api import jsonify
from rains.api import URL_PREFIX
from rains.api import Blueprint
from rains.api import Response
from rains.api import ServerParameterHandler

from rains.kit.web import *


mgr = Manager()
CORE_QUEUE: Queue = mgr.Queue(maxsize=2)
TASK_QUEUE: Queue = mgr.Queue(maxsize=20)

CORE_QUEUE.put(WebCore())
CORE_QUEUE.put(WebCore())


# 任务接口蓝图
rains_blueprint = Blueprint('rains', __name__)


@rains_blueprint.route(f'{ URL_PREFIX }/rains/test', methods=['GET'])
def tasks() -> Response:
    """

    """
    
    TASK_QUEUE.put(TaskTest)

    return jsonify({
        'code': 0,
        'result': {
            'data': '[测试] 任务已接收',
        },
        'message': 'ok',
        'type': 'success'
    })


@rains_blueprint.route(f'{ URL_PREFIX  }/rains/run', methods=['POST'])
def get_user_info():
    try:
        paras: dict = ServerParameterHandler.analysis_request_parameter(keys=['name'])
        token = request.headers['Authorization']

        if paras['name'] == 'CBLINK':
            TASK_QUEUE.put(BdSearch)

        return jsonify({
            'code': 0,
            'result': {
                'data': f'[{ paras["name"] }] 任务已完成'
            },
            'message': 'ok',
            'type': 'success'
        })

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')


rains_app.register_blueprint(rains_blueprint)


class RainsServer(object):
    """
    [ Rains 服务 ]

    * 开启时，将在本地挂起一个 Rains 执行池进程与 RainsAPI 服务。

    """

    _log: Log = Log()
    """ [ 日志服务 ] """

    _pool: Pool
    """ [ 进程池 ] """

    _pool_state: bool
    """ [ 进程池运作状态 ] """

    _max_quantity_pool: int = 3
    """ [ 进程最大阈值 ] """

    _max_quantity_core: int = 5
    """ [ 核心最大阈值 ] """

    _max_quantity_task: int = 20
    """ [ 任务最大阈值 ] """

    _lock_from_core_queue: Lock
    """ [ 核心队列互斥锁 ] """

    _lock_from_task_queue: Lock
    """ [ 任务队列互斥锁 ] """

    _lock_from_pool_state: Lock
    """ [ 进程池运作状态互斥锁 ] """

    _lock_from_task_count: Lock
    """ [ 注册的任务数量互斥锁 ] """

    def __init__(self):
        """
        [ 运行池 ]

        * 初始化。

        """

        # 初始化赋值
        self._initialization()

    def _initialization(self):
        """
        [ 初始化进程池 ]

        * NOT MESSAGE

        """

        # 创建互斥锁
        self._lock_from_core_queue = mgr.Lock()
        self._lock_from_task_queue = mgr.Lock()
        self._lock_from_pool_state = mgr.Lock()
        self._lock_from_task_count = mgr.Lock()

        # 创建进程池
        self._pool = Pool(self._max_quantity_pool)

    def running(self):
        """
        [ 运行 ]

        * NOT MESSAGE

        """

        self._pool.apply_async(func=self.monitor)

        http_server = HTTPServer(WSGIContainer(rains_app))
        http_server.listen(3700)
        print('RainsAPI running...')
        IOLoop.current().start()

    def monitor(self):
        """
        [ 监听 ]

        * NOT MESSAGE

        """

        self._pool_state = True

        while True:

            time.sleep(3)
            print('监听周期开始')

            task = TASK_QUEUE.get()

            # 驱动冷启动时会占用大量的 CPU 资源，为避免因为 CPU 占用率过高导致进程启动异常
            # 每隔 2 秒检查 CPU 占用率，只有 CPU 占用率少于 50 时才会启动进程
            while True:
                if psutil.cpu_percent(2) < 50:
                    print('调用进程执行任务..')
                    self._pool.apply_async(func=self._emit, args=[task,])
                    break

    def _emit(self, task):
        """
        [ 触发工作流 ]

        * NOT MESSAGE

        """

        # 强制 GC 回收垃圾避免内存溢出
        gc.collect()

        # 取出队列中的核心与任务
        core = CORE_QUEUE.get()

        # 执行核心的 running 函数
        core.start_task(task)

        # 触发回调
        self._function_call_back()

        # 核心执行完任务后，放回核心队列等待
        with self._lock_from_core_queue:
            CORE_QUEUE.put(core)

    def _function_call_back(self):
        """
        [ 触发回调 ]

        * 该回调将消耗任务数量。

        """

        with self._lock_from_task_count:
            ...


class TaskTest(WebTask):
    """
    测试任务
    """
    user_input: WebElement
    password_input: WebElement
    login_button: WebElement
    create_button: WebElement

    def set_class_starting(self):
        self.user_input = self.plant.element(
            page='超管后台',
            name='邮箱输入框',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div[2]/form/div[1]/div/input'
        )

        self.password_input = self.plant.element(
            page='超管后台',
            name='密码输入框',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div[2]/form/div[2]/div/input'
        )

        self.login_button = self.plant.element(
            page='超管后台',
            name='Login按钮',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div[2]/form/div[4]/div/button'
        )

        self.create_button = self.plant.element(
            page='超管后台',
            name='Login按钮',
            by_key=BY.XPATH,
            by_value='/html/body/div/div/div/div/div/div[1]/a'
        )

    def set_function_starting(self):
        self.plant.view.to_url('http://manage.dev-tea.cblink.net/')

    def case_01(self):
        """
        登录成功
        """
        self.user_input.input.send('admin@baocai.us')
        self.password_input.input.send('123456')
        self.login_button.mouse.tap()
        return self.create_button.get.in_page() is True


class BdSearch(WebTask):
    """
    测试任务
    """
    search_input: WebElement
    search_ok: WebElement

    def set_class_starting(self):
        self.search_input = self.plant.element(
            page='百度',
            name='搜索输入框',
            by_key=BY.XPATH,
            by_value='//*[@id="kw"]'
        )

        self.search_ok = self.plant.element(
            page='百度',
            name='搜索确认按钮',
            by_key=BY.XPATH,
            by_value='//*[@id="su"]'
        )

    def set_function_starting(self):
        self.plant.view.to_url('http://www.baidu.com/')

    def case_01(self):
        """
        搜索
        """
        self.search_input.input.send('CBLINK')
        self.search_ok.mouse.tap()
        time.sleep(2)
        return True



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


class ConstDbTableNaming(object):
    """
    [ 常量 :: 数据库表命名 ]

    * NOT MESSAGE

    """

    RUN_ENVIRONMENT: str = 'run_environment'
    """ [ 常量 :: 数据库表命名 :: 运行环境表 ] """

    CONFIG: str = 'configs'
    """ [ 常量 :: 数据库表命名 :: 配置表 ] """

    TASKS: str = 'tasks'
    """ [ 常量 :: 数据库表命名 :: 任务表 ] """

    CASES: str = 'cases'
    """ [ 常量 :: 数据库表命名 :: 用例表 ] """


class ConstDbRunEnvironmentNaming(object):
    """
    [ 常量 :: 运行环境表字段命名 ]

    * NOT MESSAGE

    """

    POOL_RUN_STATE: str = 'pool_run_state'
    """ [ 常量 :: 运行环境表字段命名 :: 进程池运行状态 ] """

    CORE_SIGN_COUNT: str = 'core_sign_count'
    """ [ 常量 :: 运行环境表字段命名 :: 核心注册数量 ] """

    TASK_SIGN_COUNT: str = 'task_sign_count'
    """ [ 常量 :: 运行环境表字段命名 :: 任务注册数量 ] """


class ConstDbConfigNaming(object):
    """
    [ 常量 :: 配置表字段命名 ]

    * NOT MESSAGE

    """

    ID: str = 'id'
    """ [ 常量 :: 配置表字段命名 :: 配置编号 ] """

    NAME: str = 'name'
    """ [ 常量 :: 配置表字段命名 :: 配置名称 ] """

    CORE_MAXSIZE: str = 'core_maxsize'
    """ [ 常量 :: 配置表字段命名 :: 核心最大值 ] """

    TASK_MAXSIZE: str = 'task_maxsize'
    """ [ 常量 :: 配置表字段命名 :: 任务最大值 ] """


class ConstDbTaskNaming(object):
    """
    [ 常量 :: 任务表字段命名 ]

    * NOT MESSAGE

    """

    TID: str = 'tid'
    """ [ 常量 :: 任务表字段命名 :: 任务编号 ] """

    NAME: str = 'name'
    """ [ 常量 :: 任务表字段命名 :: 任务名称 ] """

    REMARK: str = 'remark'
    """ [ 常量 :: 任务表字段命名 :: 任务备注信息 ] """

    CREATED_DATE: str = 'created_date'
    """ [ 常量 :: 任务表字段命名 :: 创建日期 ] """

    STATE: str = 'state'
    """ [ 常量 :: 任务表字段命名 :: 任务状态 ] """

    START_TIME: str = 'start_time'
    """ [ 常量 :: 任务表字段命名 :: 开始时间 ] """

    END_TIME: str = 'end_time'
    """ [ 常量 :: 任务表字段命名 :: 结束时间 ] """

    SPEND_TIME_S: str = 'spend_time_s'
    """ [ 常量 :: 任务表字段命名 :: 消耗时间(秒) ] """

    CASE_ALL: str = 'case_all'
    """ [ 常量 :: 任务表字段命名 :: 所有用例计数 ] """

    CASE_PASS: str = 'case_pass'
    """ [ 常量 :: 任务表字段命名 :: 成功的用例计数 ] """

    CASE_FAIL: str = 'case_fail'
    """ [ 常量 :: 任务表字段命名 :: 失败的用例计数 ] """


class ConstDbCaseNaming(object):
    """
    [ 常量 :: 用例表字段命名 ]

    * NOT MESSAGE

    """

    CID: str = 'cid'
    """ [ 常量 :: 用例表字段命名 :: 用例编号 ] """

    TID: str = 'tid'
    """ [ 常量 :: 用例表字段命名 :: 所属任务编号 ] """

    NAME: str = 'name'
    """ [ 常量 :: 用例表字段命名 :: 用例名称 ] """

    REMARK: str = 'remark'
    """ [ 常量 :: 用例表字段命名 :: 用例备注信息 ] """

    STATE: str = 'state'
    """ [ 常量 :: 用例表字段命名 :: 用例状态 ] """

    CREATED_DATE: str = 'created_date'
    """ [ 常量 :: 用例表字段命名 :: 创建日期 ] """

    START_TIME: str = 'start_time'
    """ [ 常量 :: 用例表字段命名 :: 开始时间 ] """

    END_TIME: str = 'end_time'
    """ [ 常量 :: 用例表字段命名 :: 结束时间 ] """

    SPEND_TIME_S: str = 'spend_time_s'
    """ [ 常量 :: 用例表字段命名 :: 消耗时间(秒) ] """

    RUN_STEP: str = 'run_step'
    """ [ 常量 :: 用例表字段命名 :: 运行步骤 ] """



# !/usr/bin/env python
# coding:utf-8

"""
[ API 蓝图模块 ]

* NOT MESSAGE

"""

from .api_test import test_blueprint
from .api_run_environment import run_environment_blueprint


__all__ = [
    test_blueprint,
    run_environment_blueprint,
]



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


import math

from rains.api.common import *


# 任务接口蓝图
task_blueprint = Blueprint('task', __name__)


@task_blueprint.route('/task/tasks', methods=['POST'])
def tasks() -> Response:
    """
    [ 获取任务列表 ]
    * 无

    [ 必要参数 ]
    * 无

    [ 可选参数 ]
    * page (int): 页数
    * number (int): 查询数量

    [ 返回内容 ]
    * (Response) : 任务信息列表

    """

    try:
        # 解析请求参数
        paras = ServerParameterHandler.analysis_request_parameter('page', 'number')
        # 获取服务器数据
        task_list = DB.read(SQL.task.get_info_all(paras))
        task_all_count = DB.read(SQL.task.get_count_all())[0][0]

        # 加工数据
        return_data = {'tasks': task_list}
        page = 1 if 'page' not in paras.keys() else int(paras['page'])
        number = 10 if 'number' not in paras.keys() else int(paras['number'])
        return_data.update({'current_page': page})
        # 获取所有的页面
        all_page = math.ceil(task_all_count / number)
        return_data.update({'all_page': all_page})
        return_data.update({'next_page': page + 1 if page < all_page else page})
        return_data.update({'back_page': page - 1 if page > 1 else 1})

        return ServerParameterHandler.successful(return_data)

    except BaseException as e:
        return ServerParameterHandler.unsuccessful(f'{ e }')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

__all__ = [
    'RAINS_DB',
    'RAINS_SQL',
    'URL_PREFIX',
    'Response',
    'Blueprint',
    'ServerParameterHandler',
]

import json

from flask import jsonify
from flask import request
from flask import Response
from flask import Blueprint

from rains.db.rains_db import RainsDb
from rains.db.rains_sql import RainsSql
from rains.const.const_api import ConstApiStateMsg
from rains.const.const_api import ConstApiStateCode


RAINS_DB: RainsDb = RainsDb()
""" [ 数据库实例 ] """

RAINS_SQL: RainsSql = RainsSql()
""" [ SQL语句实例 ] """

URL_PREFIX: str = '/api'
""" [ URL 统一前缀 ] """


class ServerParameterHandler(object):
    """
    [ 服务端参数处理程序 ]

    * 该类用于解析前端请求参数，以及拼接返回给前端的 Json 数据结构。

    """

    def __init__(self):
        """
        [ 服务端参数处理程序 ]

        * 初始化。

        """

        ...

    @staticmethod
    def analysis_request_parameter(keys: list, must_keys: list = []) -> dict:
        """
        [ 解析请求参数 ]

        * 从请求中解析指定参数。
        * 如果解析得到的参数值为空，则舍弃该参数。
        * 如果 must_keys 不为空列表，则检查解析结果是否包含 must_keys。

        Args:
            keys (list): [ 需解析的参数 Key 列表 ]
            must_keys (list): [ 必须包含的 Key 列表 ]

        Raises:
            ServerAnalysisParameterError: [ 服务端解析参数异常 ]

        Returns:
            dict: [ 返回解析完成的参数字典 ]

        """

        try:
            request_handle = request.args
            if request.method == 'POST':
                request_handle = json.loads(request.data.decode('UTF-8'))

            decryption_paras: dict = {}
            for key in keys:
                value = request_handle.get(key)
                if not value: 
                    ...  
                else:
                    decryption_paras.update({key: value})
            
            if len(must_keys) > 0:
                ServerParameterHandler.inspection_parameter(decryption_paras, must_keys)

        except BaseException as err:
            raise ServerAnalysisParameterError(str(err))

        else:
            return decryption_paras
    
    @staticmethod
    def inspection_parameter(paras: dict, keys: list):
        """
        [ 检查参数是否包含必要 Keys ]

        * 如果 keys 的所有元素都存在于 paras 中，则通过，否则抛出 ServerAnalysisParameterError 异常。

        Args:
            paras (dict): [ 参数字典 ]
            keys (list): [ Key 列表 ]

        Raises:
            ParametersAreMissingException: [description]

        """

        for key in keys:
            try:
                if not paras[key]:
                    ...

            except KeyError as err:
                raise ServerAnalysisParameterError(str(err))

    @staticmethod
    def successful(paras: dict) -> jsonify:
        """
        [ 请求成功 ]

        * NOT MESSAGE

        Args:
            paras (dict): [ 需要返回的数据字典 ]

        Returns:
            jsonify: [ 返回前端的 Json 数据结构 ]

        """

        return jsonify({
            'code': ConstApiStateCode.SUCCESSFUL,
            'message': ConstApiStateMsg.SUCCESSFUL,
            'result': paras
        })

    @staticmethod
    def unsuccessful(err: str) -> jsonify:
        """
        [ 请求失败 ]

        * NOT MESSAGE

        Args:
            err (str): [ 需要返回的错误信息 ]

        Returns:
            jsonify: [ 返回前端的 Json 数据结构 ]

        """

        return jsonify({
            'code': ConstApiStateCode.UNSUCCESSFUL,
            'message': ConstApiStateMsg.UNSUCCESSFUL,
            'err': err
        })


class ServerAnalysisParameterError(Exception):

    def __init__(self, message: str):
        """
        [ 服务端解析参数异常 ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的错误信息 ]

        """

        self.message = message
    
    def __str__(self) -> str:
        
        return f"服务端解析参数异常:: { self.message }"



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

import sqlite3

from threading import Lock

from rains.const.const_file import ConstPath
from rains.db.rains_sql import RainsSql
from rains.const.const_db import ConstDbTaskNaming
from rains.const.const_db import ConstDbCaseNaming
from rains.const.const_db import ConstDbTableNaming
from rains.const.const_db import ConstDbConfigNaming
from rains.const.const_db import ConstDbRunEnvironmentNaming

from rains.common.decorator import singleton_pattern


@singleton_pattern
class RainsDb(object):
    """
    [ Rains 数据库 ]

    * 基于 SQLite 数据库。

    """

    _conn: sqlite3.Connection = None
    """ [ 数据库连接对象 ] """

    _lock: Lock = Lock()
    """ [ 数据库互斥锁 ] """

    def __init__(self):
        """
        [ Rains 数据库 ]

        * 初始化。

        """

        try:
            # 创建当前项目的日志目录
            if not ConstPath.DATA.is_dir():
                ConstPath.DATA.mkdir(parents=True, exist_ok=True)

            # 创建/连接数据库
            if not self._conn:
                self.connect()
                self._conn.commit()

        except BaseException as err:
            raise RainsDbInitError(str(err))

    def read(self, sql: str) -> list:
        """
        [ 无锁读取 ]

        * 执行查询语句，该函数不会持有数据库锁。

        Args:
            sql (str): [ SQLite 查询语句 ]

        Raises:
            RainsDbReadError: [ Rains 数据库读取异常 ]

        Returns:
            list: [ 查询结果列表 ]

        """

        try:
            # 创建数据库游标
            cur = self._conn.cursor()
            # 执行查询语句
            cur.execute(sql)
            # 获取查询结果
            data = cur.fetchall()
            # 注销数据库游标
            cur.close()

            return data

        except BaseException as err:
            raise RainsDbReadError(str(err))

    def lock_read(self, sql: str) -> list:
        """
        [ 持锁读取 ]

        * 执行查询语句，该函数会持有数据库锁，结束时释放。

        Args:
            sql (str): [ SQLite 查询语句 ]

        Raises:
            RainsDbReadError: [ Rains 数据库读取异常 ]

        Returns:
            list: [ 查询结果列表 ]

        """

        # 获取锁
        with self._lock:

            # 执行读取函数
            return self.read(sql)

    def write(self, sql: str) -> int:
        """
        [ 无锁写入 ]

        * 执行写入语句，该函数不会持有数据库锁。

        Args:
            sql (str): [ SQLite 语句 ]

        Raises:
            RainsDbWriteError: [ Rains 数据库写入异常 ]

        Returns:
            int: [ 写入数据后返回该条数据的自增ID ]

        """

        try:
            # 创建数据库游标
            cur = self._conn.cursor()
            # 执行查询语句
            cur.execute(sql)
            # 获取数据自增ID
            aid = cur.lastrowid
            # 注销数据库游标
            cur.close()

            return aid

        except BaseException as err:
            raise RainsDbWriteError(str(err))
    
    def lock_write(self, sql: str) -> int:
        """
        [ 持锁写入 ]

        * 执行写入语句，该函数会持有数据库锁，结束时释放。

        Args:
            sql (str): [ SQLite 语句 ]

        Raises:
            RainsDbWriteError: [ Rains 数据库写入异常 ]

        Returns:
            int: [ 写入数据后返回该条数据的自增ID ]
        """

        # 获取锁
        with self._lock:

            # 执行写入函数
            return self.write(sql)
        

    def connect(self):
        """
        [ 数据库连接 ]

        * NOT MESSAGE

        """

        # 创建数据库连接对象
        print(ConstPath.DATA_DB)
        self._conn = sqlite3.connect(ConstPath.DATA_DB, check_same_thread=False)
        # 创建数据库游标
        Cur = self._conn.cursor()

        # 构建数据库表
        for key, value in CreateTableSql.__dict__.items():
            if '__' in key:
                continue
            Cur.execute(value)

        # 清空运行环境表数据
        Cur.execute(f'DELETE FROM { ConstDbTableNaming.RUN_ENVIRONMENT }')
        Cur.execute(f'DELETE FROM { ConstDbTableNaming.CONFIG }')

        # 初始化运行环境表数据
        self.lock_write(RainsSql().run_environment.init())

        # 注销数据库游标
        Cur.close()

    def commit(self):
        """
        [ 事务保存 ]

        * NOT MESSAGE

        """

        self._conn.commit()

    def rollback(self):
        """
        [ 事务回滚 ]

        * NOT MESSAGE

        """

        self._conn.rollback()

    def quit(self):
        """
        [ 关闭数据库连接 ]

        * NOT MESSAGE

        """

        self._conn.close()


class CreateTableSql(object):
    """
    [ 建表语句 ]

    * NOT MESSAGE

    """

    RUN_ENVIRONMENT = f"""

    CREATE TABLE IF NOT EXISTS { ConstDbTableNaming.RUN_ENVIRONMENT }
    (
        id                                               INTEGER PRIMARY KEY,
        { ConstDbRunEnvironmentNaming.POOL_RUN_STATE }   BLOB NOT NULL,
        { ConstDbRunEnvironmentNaming.CORE_SIGN_COUNT }  INT NOT NULL,
        { ConstDbRunEnvironmentNaming.TASK_SIGN_COUNT }  INT NOT NULL
    );

    """

    """
    [ 建表语句 :: 运行环境表 ]

    * 记录当前的运行环境。

    Structure:
        id (int): 环境编号，自增的表主键
        pool_run_state (bool): 进程池运行状态
        core_sign_count (int): 核心注册数量
        task_sign_count (int): 任务注册数量

    """

    CONFIGS = f"""

    CREATE TABLE IF NOT EXISTS { ConstDbTableNaming.CONFIG }
    (
        { ConstDbConfigNaming.ID }            INTEGER PRIMARY KEY,
        { ConstDbConfigNaming.NAME }          TEXT NOT NULL,
        { ConstDbConfigNaming.CORE_MAXSIZE }  INT NOT NULL,
        { ConstDbConfigNaming.TASK_MAXSIZE }  INT NOT NULL
    );

    """

    """
    [ 建表语句 :: 配置表 ]

    * 记录配置信息。

    Structure:
        id (int): 配置编号，自增的表主键
        name (str): 配置名称
        core_maxsize (int): 核心最大值
        task_maxsize (int): 任务最大值

    """

    TASKS = f""" 

    CREATE TABLE IF NOT EXISTS { ConstDbTableNaming.TASKS }
    (
        { ConstDbTaskNaming.TID }           INTEGER PRIMARY KEY,
        { ConstDbTaskNaming.NAME }          TEXT NOT NULL,
        { ConstDbTaskNaming.REMARK }        TEXT NOT NULL,
        { ConstDbTaskNaming.CREATED_DATE }  DATE NOT NULL,
        { ConstDbTaskNaming.STATE }         TEXT NOT NULL,
        { ConstDbTaskNaming.START_TIME }    DATE,
        { ConstDbTaskNaming.END_TIME }      DATE,
        { ConstDbTaskNaming.SPEND_TIME_S }  INT,
        { ConstDbTaskNaming.CASE_ALL }      INT,
        { ConstDbTaskNaming.CASE_PASS }     INT,
        { ConstDbTaskNaming.CASE_FAIL }     INT
    );

    """

    """
    [ 建表语句 :: 任务表 ]

    * 记录任务的执行信息。

    Structure:
        tid (int): 任务编号，自增的表主键
        name (str): 任务名称
        remark (str): 任务备注信息
        created_date (date): 创建日期
        state (bool): 任务状态
        start_time (date): 开始时间
        end_time (date): 结束时间
        spend_time_s (date): 消耗时间(秒)
        case_all (int): 所有用例计数
        case_pass (int): 成功的用例计数
        case_fail (int): 失败用例数计数

    """

    CASES = f"""

    CREATE TABLE IF NOT EXISTS { ConstDbTableNaming.CASES }
    (
        { ConstDbCaseNaming.CID }           INTEGER PRIMARY KEY,
        { ConstDbCaseNaming.TID }           INT NOT NULL,
        { ConstDbCaseNaming.NAME }          TEXT NOT NULL,
        { ConstDbCaseNaming.REMARK }        TEXT NOT NULL,
        { ConstDbCaseNaming.STATE }         TEXT NOT NULL,
        { ConstDbCaseNaming.CREATED_DATE }  DATE NOT NULL,
        { ConstDbCaseNaming.START_TIME }    DATE,
        { ConstDbCaseNaming.END_TIME }      DATE,
        { ConstDbCaseNaming.SPEND_TIME_S }  INTEGER,
        { ConstDbCaseNaming.RUN_STEP }      TEXT
    );
        
    """

    """
    [ 建表语句 :: 用例表 ]

    * 记录用例的执行信息。
    
    Structure:
        cid (int): 用例编号，自增的表主键
        tid (int): 所属的任务记录编号
        name (str): 用例名称
        remark (str): 用例备注信息
        state (str): 用例状态
        created_date (date): 创建日期
        start_time (date): 开始时间
        end_time (date): 结束时间
        spend_time_s (date): 消耗时间(秒)
        run_step (str): 运行步骤
            
    """


class RainsDbInitError(Exception):

    def __init__(self, message: str):
        """
        [ Rains 数据库初始化异常 ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的错误信息 ]

        """

        self.message = message
    
    def __str__(self) -> str:
        
        return f"Rains 数据库初始化异常:: { self.message }"


class RainsDbReadError(Exception):

    def __init__(self, message: str):
        """
        [ Rains 数据库读取异常 ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的错误信息 ]

        """

        self.message = message
    
    def __str__(self) -> str:
        
        return f"Rains 数据库读取异常:: { self.message }"


class RainsDbWriteError(Exception):

    def __init__(self, message: str):
        """
        [ Rains 数据库写入异常 ]

        * NOT MESSAGE

        Args:
            message (str): [ 输出的错误信息 ]

        """

        self.message = message
    
    def __str__(self) -> str:
        
        return f"Rains 数据库写入异常:: { self.message }"
